import os
import shutil
from http.server import HTTPServer, BaseHTTPRequestHandler
from multiprocessing import Process
from importlib import reload

import prometheus_client
from prometheus_client import generate_latest, REGISTRY, multiprocess, CollectorRegistry, Counter

from lemmings.utils.influx import Influx

web_requests = Counter(f'web_requests', f'Latency for incoming web requests', ["url"])
class TestServer(BaseHTTPRequestHandler):
    def do_GET(self):
        web_requests.labels(url="/prometheus").inc()
        self.send_response(200)
        self.send_header('Content-type', 'text/plain')
        self.end_headers()
        self.wfile.write(generate_latest(REGISTRY))

def run_server(server_class=HTTPServer, handler_class=TestServer):
    reload(prometheus_client)
    multiprocess.MultiProcessCollector(REGISTRY)

    server_address = ('', 8000)
    httpd = server_class(server_address, handler_class)
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass
    httpd.server_close()
    print("stop server")


ENV = 'prometheus_multiproc_dir'

class Prometheus:
    def __init__(self, registry=REGISTRY, shared_dir='./prometheus.tmp'):
        if ENV in os.environ:
            self.path = os.environ.get(ENV)
        else:
            self.path = os.environ.setdefault(ENV, shared_dir)
        shutil.rmtree(self.path, ignore_errors=True)
        os.mkdir(self.path)

        if not registry:
            registry = CollectorRegistry()
        self.registry = registry
        multiprocess.MultiProcessCollector(self.registry)
        self.influx = Influx(self.registry)
        self.args = []

    def filter(self, *args):
        self.args = args

    def dump_to_influx(self, all=False):
        args = self.args if not all else []
        return self.influx.save(*args)

    def clean(self):
        try:
            shutil.rmtree(self.path)
            print("temporary prometheus dir cleared")
            self.prom_server.terminate()
            print("prometheus thread terminated")
            self.prom_server.join(5)
            print("prometheus thread stopped")

        except BaseException as e:
            print("problem with temporary prometheus dir: ", e)

    def start_server_process(self):
        self.prom_server = Process(target=run_server, args=())
        self.prom_server.start()




import asyncio
import concurrent
import functools
import time
from datetime import timedelta

from lemmings.utils.parsing import parse_duration


def duration2sec(duration):
    """
    convert duration to seconds (as float)
    :param duration: seconds, timedelta or formatted string
    :return:
    """
    if isinstance(duration, str):
        duration = parse_duration(duration)
    if isinstance(duration, timedelta):
        duration = duration.total_seconds()
    return duration


def async_timeout(duration):
    """
    timeout decorator for async function
    :param duration:
    :return:
    """

    def wrap(func):
        if asyncio.iscoroutinefunction(func):
            @functools.wraps(func)
            async def wrapper_timer(*args, **kwargs):
                start = time.time()
                try:
                    return await asyncio.wait_for(func(*args, **kwargs), duration2sec(duration))
                except concurrent.futures._base.TimeoutError as e:
                    actual = timedelta(seconds=time.time() - start)
                    print(f"timeout during {func.__name__}({args}) [after: {actual}, max: {duration}]")
                    return None
            return wrapper_timer
    return wrap


def in_thread_executor(f):
    """
    Convert func to async: run sync code in thread executor
    WARNING! Code could not be cancelled by async helpers
    :param f:
    :return:
    """

    @functools.wraps(f)
    async def wrap(*args):
        ts = time.time()
        # None uses the default executor (ThreadPoolExecutor)
        result = await asyncio.get_event_loop().run_in_executor(None, f, *args)
        te = time.time()
        print('* %s tooks: %2.4f sec. \n - args: %r' % (f.__name__.upper(), te - ts, args))
        return result

    return wrap
